#undef TRACE_SYSTEM
#define TRACE_SYSTEM neigh

#if !defined(_TRACE_NEIGH_H) || defined(TRACE_HEADER_MULTI_READ)
#define _TRACE_NEIGH_H

#include <linux/skbuff.h>
#include <linux/netdevice.h>
#include <linux/tracepoint.h>
#include <net/neighbour.h>

TRACE_EVENT(neigh_notify,

	TP_PROTO(struct neighbour *n, int type, int flags, u32 pid),

	TP_ARGS(n, type, flags, pid),

	TP_STRUCT__entry(
		__field(u32, family)
		__string(dev, n->dev->name)
		__array(u8, lladdr, MAX_ADDR_LEN)
		__field(u8, flags)
		__field(u8, nud_state)
		__field(u8, type)
		__field(u8, dead)
		__field(int, refcnt)
		__array(__u8, primary_key4, 4)
		__array(__u8, primary_key6, 16)
		__field(u32, nl_type)
		__field(u32, nl_flags)
		__field(u32, nl_pid)
	),

	TP_fast_assign(
		struct in6_addr *pin6;
		__be32 *p32;

		__entry->family = n->tbl->family;
		__assign_str(dev, n->dev->name);
		memcpy(__entry->lladdr, n->ha, n->dev->addr_len);
		__entry->flags = n->flags;
		__entry->nud_state = n->nud_state;
		__entry->type = n->type;
		__entry->dead = n->dead;
		__entry->refcnt = atomic_read(&n->refcnt);
		__entry->nl_type = type;
		__entry->nl_flags = flags;
		__entry->nl_pid = pid;
		pin6 = (struct in6_addr *)__entry->primary_key6;
		p32 = (__be32 *)__entry->primary_key4;

		if (n->tbl->family == AF_INET)
			*p32 = *(__be32 *)n->primary_key;
		else
			*p32 = 0;

#if IS_ENABLED(CONFIG_IPV6)
		if (n->tbl->family == AF_INET6) {
			pin6 = (struct in6_addr *)__entry->primary_key6;
			*pin6 = *(struct in6_addr *)n->primary_key;
		} else
#endif
		{
			ipv6_addr_set_v4mapped(*p32, pin6);
		}

	),

	TP_printk("family %d dev %s lladdr %02x:%02x:%02x:%02x:%02x:%02x flags %02x nud_state %02x type %02x dead %d refcnt %d primary_key4 %pI4 primary_key6 %pI6c nl_type %d nl_flags %d nl_pid %d",
		  __entry->family, __get_str(dev), __entry->lladdr[0], __entry->lladdr[1],
		  __entry->lladdr[2], __entry->lladdr[3], __entry->lladdr[4],
		  __entry->lladdr[5], __entry->flags, __entry->nud_state,
		  __entry->type, __entry->dead, __entry->refcnt,
		  __entry->primary_key4, __entry->primary_key6,
		  __entry->nl_type, __entry->nl_flags, __entry->nl_pid)
);

#endif /* _TRACE_NEIGH_H */

/* This part must be outside protection */
#include <trace/define_trace.h>
